import { Responses, Requests, post, post_token } from "./base";

export const auth_login = (
  username: string,
  password: string,
  otp?: string
): Promise<
  | Responses.Auth.LoginResponse
  | Responses.Auth.TfaRequiredResponse
  | Responses.ErrorResponse
> =>
  post<Requests.Auth.LoginRequest>("/api/auth/login", {
    username: username,
    password: password,
    otp: otp,
  });

export const auth_signup = (
  username: string,
  password: string
): Promise<Responses.Auth.SignupResponse | Responses.ErrorResponse> =>
  post<Requests.Auth.SignUpRequest>("/api/auth/signup", {
    username: username,
    password: password,
  });

export const refresh_token = (
  token: string
): Promise<Responses.Auth.RefreshResponse | Responses.ErrorResponse> =>
  post_token("/api/auth/refresh", {}, token);

export const change_password = (
  oldPw: string,
  newPw: string,
  token: string
): Promise<Responses.Auth.ChangePasswordResponse | Responses.ErrorResponse> =>
  post_token<Requests.Auth.ChangePasswordRequest>(
    "/api/auth/change_password",
    {
      oldPassword: oldPw,
      newPassword: newPw,
    },
    token
  );

export const logout_all = (
  token: string
): Promise<Responses.Auth.LogoutAllResponse | Responses.ErrorResponse> =>
  post_token("/api/auth/logout_all", {}, token);

export function tfa_setup(
  mail: false,
  token: string
): Promise<Responses.Auth.RequestTotpTfaResponse | Responses.ErrorResponse>;
export function tfa_setup(
  mail: true,
  token: string
): Promise<Responses.Auth.RequestEmailTfaResponse | Responses.ErrorResponse>;
export function tfa_setup(
  mail: boolean,
  token: string
): Promise<
  | Responses.Auth.RequestEmailTfaResponse
  | Responses.Auth.RequestTotpTfaResponse
  | Responses.ErrorResponse
> {
  return post_token<Requests.Auth.TfaSetup>(
    "/api/auth/2fa/setup",
    {
      mail,
    },
    token
  );
}

export const tfa_complete = (
  mail: boolean,
  code: string,
  token: string
): Promise<Responses.Auth.TfaCompletedResponse | Responses.ErrorResponse> =>
  post_token<Requests.Auth.TfaComplete>(
    "/api/auth/2fa/complete",
    {
      mail,
      code,
    },
    token
  );

export const tfa_disable = (
  token: string
): Promise<Responses.Auth.RemoveTfaResponse | Responses.ErrorResponse> =>
  post_token("/api/auth/2fa/disable", {}, token);
